package cc.fascinated.bat.features.spotify.command;

import cc.fascinated.bat.Emojis;
import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.spotify.SpotifyFeature;
import cc.fascinated.bat.features.spotify.profile.SpotifyProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.SpotifyService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import se.michaelthelin.spotify.model_objects.miscellaneous.CurrentlyPlaying;
import se.michaelthelin.spotify.model_objects.specification.Track;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(name = "skip", description = "Skip the current Spotify track")
public class SkipSubCommand extends BatSubCommand {
    private static final Logger log = LoggerFactory.getLogger(SkipSubCommand.class);
    private final SpotifyService spotifyService;

    @Autowired
    public SkipSubCommand(@NonNull SpotifyService spotifyService) {
        this.spotifyService = spotifyService;
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction interaction) {
        SpotifyProfile profile = user.getProfile(SpotifyProfile.class);
        if (!profile.hasLinkedAccount()) {
            interaction.replyEmbeds(SpotifyFeature.linkAccountEmbed()).queue();
            return;
        }

        if (!spotifyService.hasTrackPlaying(user)) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("%s You need to be playing a track to skip a track.".formatted(Emojis.CROSS_MARK_EMOJI))
                            .build())
                    .queue();
            return;
        }

        CurrentlyPlaying currentlyPlaying = spotifyService.getCurrentlyPlayingTrack(user);
        Track track = (Track) currentlyPlaying.getItem();
        String trackName = track.getName();

        spotifyService.skipTrack(user);
        Track newTrack = getNewTrack(user, trackName);
        interaction.replyEmbeds(EmbedUtils.successEmbed()
                        .setDescription("""
                                :track_next: Skipped the track: **[%s | %s](%s)**
                                %s New Track: **[%s | %s](%s)**
                                """.formatted(
                                trackName,
                                track.getArtists()[0].getName(),
                                "https://open.spotify.com/track/" + track.getId(),
                                Emojis.CHECK_MARK_EMOJI,
                                newTrack.getName(),
                                newTrack.getArtists()[0].getName(),
                                "https://open.spotify.com/track/" + newTrack.getId()
                        )).build())
                .queue();
    }

    /**
     * Get the next track that is playing
     *
     * @param user    The user to get the track for
     * @param oldName The name of the old track
     * @return The new track
     */
    public Track getNewTrack(BatUser user, String oldName) {
        int checks = 0;

        try {
            Thread.sleep(150);
            while (checks < 10) {
                CurrentlyPlaying currentlyPlaying = spotifyService.getCurrentlyPlayingTrack(user);
                Track track = (Track) currentlyPlaying.getItem();
                if (track.getName().equals(oldName)) {
                    Thread.sleep(250);
                    checks++;
                } else {
                    log.info("Found new track {} in {} checks", track.getName(), checks);
                    return (Track) currentlyPlaying.getItem();
                }
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return null;
    }
}
